package ca.ubc.cs211.mp3.ui;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.SortedSet;

import ca.ubc.cs211.mp3.library.*;
import ca.ubc.cs211.mp3.util.StringFormat;
import ca.ubc.cs211.player.MP3;

// Simple text interface and feature-limited MP3 player

public class SimpleTextInterface {

	// used for keyboard input
	public static Scanner keyboard = new Scanner(System.in);
	
	// the library object we're operating on
	public static Library testLibrary = new Library();
	
	// references the currently toggled MP3 for playback purposes
	public static MP3 playerMP3;
	public static Song currentSong = null;

	
	public static void mainMenu() {
		while (true) {
			System.out.println("Music Library Text Interface - Main Menu");
			System.out.println("Currently toggled for playback: " + (currentSong == null ? "(nothing)" : currentSong.getTitle()));
			System.out.println("-----------------------------------------------");
			System.out.println(" 1. Import library from text file");
			System.out.println(" 2. Song view (song menu)");
			System.out.println(" 3. Artist view (artist menu)");
			System.out.println(" 4. Album view (album menu)");
			System.out.println("-----------------------------------------------");
			System.out.println("Player controls:");
			System.out.println(" 5. Play/pause (" + (currentSong == null ? "0:00" : StringFormat.formatTime(playerMP3.getPlaybackPositionSeconds())) + ")");
			System.out.println(" 6. Stop");
			System.out.println("-----------------------------------------------");
			System.out.println("99. Quit");
			System.out.println("-----------------------------------------------");
			System.out.print("-->");

			switch (keyboard.nextInt()) {
				case 1: importLibrary(); break;
				case 2: while (songMenu() != -1); break;
				case 3: while (artistMenu() != -1); break;
				case 4: while (albumMenu() != -1); break;
				case 5: playPause(); break;
				case 6: stop(); break;					
				case 99: return; 
				default : break;
			}
			
		}
	}

	public static void importLibrary() {
		testLibrary = new Library();
		System.out.println("Enter complete path to text file: ");
		keyboard.nextLine();
		String path = keyboard.nextLine();

		System.out.print("Importing library...");

		try {
			Scanner fileReader = new Scanner(new File(path));
			fileReader.nextLine(); // # SONGS
			
			// read in all songs until we read in the ARTISTS header as title
			while (true) {
				String title = fileReader.nextLine();
				if (title.equals("# ARTISTS"))
					break;
				String songPath = fileReader.nextLine();
				int track = Integer.parseInt(fileReader.nextLine());
				testLibrary.addSong(title, songPath, track);
			}
			
			// read in all artists until we read in the ALBUMS header as name
			while (true) {
				String name = fileReader.nextLine();
				if (name.equals("# ALBUMS"))
					break;
				testLibrary.addArtist(name);
			}
			
			// read in all albums until we read in the SONG-ALBUM PAIRS header
			while (true) {
				String title = fileReader.nextLine();
				if (title.equals("# SONG-ALBUM PAIRS"))
					break;
				testLibrary.addAlbum(title);
			}
			
			// read in song-album pairs until we read in the SONG-ARTIST PAIRS header
			while (true) {
				String songPath = fileReader.nextLine();
				if (songPath.equals("# SONG-ARTIST PAIRS")) 
					break;
				String albumName = fileReader.nextLine();
				Song s = testLibrary.getSongByPath(songPath);
				Album a = testLibrary.getAlbumByName(albumName);
				s.setAlbum(a);				
			}
			
			// read in song-artist pairs until we read in the EOF header, then quit
			while (true) {
				String songPath = fileReader.nextLine();
				if (songPath.equals("# EOF"))
					break;
				String artistName = fileReader.nextLine();
				Song s = testLibrary.getSongByPath(songPath);
				Artist a = testLibrary.getArtistByName(artistName);
				s.addArtist(a);
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (Exception e) {
			System.out.println("Exception encountered during import -- you need to have certain functions");
			System.out.println("implemented for import to work properly.  Look at the stack trace below for");
			System.out.println("clues as to why this exception encountered.");
			e.printStackTrace();
		}

		System.out.println("done!");
	}

	public static int songMenu() {
		SortedSet<Song> songs = testLibrary.getAllSongs();
		
		if (songs == null) {
			System.out.println("**WARNING! Song list is null.  Have you implemented getAllSongs()?");
			System.out.println();
			return -1;
		}

		if (songs.size() == 0) {
			System.out.println("** No songs in library!");
		}
		else {
			System.out.println("Viewing all songs in library...");
			int i = 1;
			for (Song s : songs) {
				System.out.println(i++ + ".  " + s.getTitle());
			}
		}

		System.out.println();
		System.out.println("Menu: ");
		System.out.println("1. View/edit song details");
		System.out.println("2. Add new song to library");
		System.out.println("3. Remove song from library");
		System.out.println("4. Toggle song for playback");
		System.out.println("5. Back to main menu");
		System.out.print("-->");
		int input = keyboard.nextInt();

		if (!(input >= 1 && input <= 5))
			return 0;

		if (songs.size() == 0 && (input == 1 || input == 3))
			return 0;


		// back to main menu
		if (input == 5) 
			return -1;  

		// existing song operations
		else if (input != 2) {
			System.out.println("Enter song number: ");
			int song = keyboard.nextInt();
			if (song > songs.size()) {
				System.out.println("Invalid song number.");
				return 0;
			}
			Song[] temp = new Song[songs.size()];
			Song s = songs.toArray(temp)[song-1];
			switch (input) {
			case 1:
				while (true) {
					System.out.println("Song details:");
					System.out.println("Title: " + s.getTitle());
					System.out.println("Track: " + s.getTrack());
					System.out.println("Album: " + s.getAlbum().getTitle());
					System.out.print("Aritist(s): ");
					for (Artist a: s.getArtists())
						System.out.print(a.getName() + ", ");
					System.out.println();
					System.out.println();
					System.out.println("Menu:");
					System.out.println("1. Change title");
					System.out.println("2. Change track number");
					System.out.println("3. Set album");
					System.out.println("4. Add artist");
					System.out.println("5. Remove artist");
					System.out.println("6. Back to song menu");
					System.out.print("-->");
					int songSubMenu = keyboard.nextInt();
					if (!(songSubMenu >= 1 && input <= 6))
						continue;
					switch (songSubMenu) {
					case 1:
						keyboard.nextLine();
						System.out.print("Enter new title: ");
						s.setTitle(keyboard.nextLine());
						break;
					case 2:
						System.out.println("Enter new track number: ");
						s.setTrack(keyboard.nextInt());
						break;
					case 3:
						System.out.println("Select album from the list below (0 for new album):");
						int albumCount = 1;
						System.out.println("0. (New Album)");
						for (Album a : testLibrary.getAllAlbums()) {
							System.out.println(albumCount++ + ". " + a.getTitle());
						}
						int albumChoice = keyboard.nextInt();
						if (albumChoice == 0) {
							keyboard.nextLine();
							System.out.print("Enter new album title: ");
							Album a = testLibrary.addAlbum(keyboard.nextLine());
							s.setAlbum(a);									
						} else {
							Album[] atemp = new Album[testLibrary.getAllAlbums().size()];
							Album newAlbum = testLibrary.getAllAlbums().toArray(atemp)[albumChoice-1];
							//Album newAlbum = ((Album[])(testLibrary.getAllAlbums().toArray()))[albumChoice-1];
							s.setAlbum(newAlbum);									
						}
						break;

					case 4:
						System.out.println("Select artist from the list below (0 for new artist):");
						int artistCount = 1;
						System.out.println("0. (New Artist)");
						for (Artist a : testLibrary.getAllArtists()) {
							System.out.println(artistCount++ + ". " + a.getName());
						}
						int artistChoice = keyboard.nextInt();
						if (artistChoice > testLibrary.getAllArtists().size())
							break;
						if (artistChoice == 0) {
							System.out.print("Enter new artist name: ");
							keyboard.nextLine();
							Artist a = testLibrary.addArtist(keyboard.nextLine());
							s.addArtist(a);									
						} else {
							Artist[] tempArtists = new Artist[testLibrary.getAllArtists().size()];
							Artist newArtist = testLibrary.getAllArtists().toArray(tempArtists)[artistChoice-1];
							s.addArtist(newArtist);									
						}
						break;

					case 5:
						if (s.getArtists().size() == 0) {
							System.out.println("No artist(s) associated with this song!");
							break;
						}

						System.out.println("Select artist from the list below: ");
						artistCount = 1;
						for (Artist a : s.getArtists()) {
							System.out.println(artistCount++ + ". " + a.getName());
						}
						int artistRemovalChoice = keyboard.nextInt();
						if (artistRemovalChoice > s.getArtists().size()) {
							break;
						}
						Artist[] tempArtists = new Artist[s.getArtists().size()];
						tempArtists = s.getArtists().toArray(tempArtists);
						Artist newArtist = tempArtists[artistRemovalChoice-1];
						//Artist newArtist = ((Artist[])(testLibrary.getAllArtists().toArray()))[artistRemovalChoice-1];
						s.removeArtist(newArtist);									
						break;


					case 6: 
						return 0;
					}

				}

			case 3:
				testLibrary.removeSong(s);
				System.out.println("Song removed from library.");
				break;

			case 4:
				currentSong = s;
				try {
					playerMP3 = new MP3(s.getPath());
					System.out.println("Song toggled for playback.");
				}
				catch (Exception e) {
					System.out.println("Error toggling song -- check file path.");
					currentSong = null;
				}
				
				break;

			default: return -1;
			}
		}

		else {
			System.out.println("Adding new song to library...");
			System.out.print("Title: ");
			keyboard.nextLine();
			String title = keyboard.nextLine();
			System.out.print("Track No.: ");
			int track = keyboard.nextInt();
			System.out.print("Path to MP3 File: ");
			keyboard.nextLine();
			String path = keyboard.nextLine();
			testLibrary.addSong(title, path, track);
			System.out.println("Done!");
		}

		return 0;

	}

	public static int artistMenu() {
		SortedSet<Artist> artists = testLibrary.getAllArtists();

		if (artists == null) {
			System.out.println("**WARNING! Artist list is null.  Have you implemented getAllArtists()?");
			System.out.println();
			return -1;
		}
		
		if (artists.size() == 0) {
			System.out.println("** No artists in library!");
		}

		else {
			System.out.println("Viewing all artists in library...");
			int i = 1;
			for (Artist a : artists) {
				System.out.println(i++ + ".  " + a.getName());
			}
		}
		System.out.println("End of listing.");

		System.out.println();
		System.out.println("Menu: ");
		System.out.println("1. View all songs by artist");
		System.out.println("2. Change name of artist");
		System.out.println("3. Add artist to library");
		System.out.println("4. Remove artist from library");
		System.out.println("5. Back to main menu");
		System.out.print("-->");
		int input = keyboard.nextInt();

		if (!(input >= 1 && input <= 5))
			return 0;

		if (artists.size() == 0 && (input == 1 || input == 2 || input == 4))
			return 0;

		// back to main menu
		else if (input == 5) 
			return -1;

		// existing song operations
		else if (input != 3) {
			System.out.println("Enter artist number: ");
			int artistNo = keyboard.nextInt();
			Artist[] tempA = new Artist[artists.size()];
			tempA = artists.toArray(tempA);
			Artist a = tempA[artistNo-1];
			switch (input) {
			case 1:
				System.out.println("Songs by artist \"" + a.getName() + "\": (use song menu to edit all song details)");
				int i = 1;
				for (Song s : a.getSongs()) {
					System.out.println(i++ + ". " + s.getTitle());
				}
				System.out.println("End of song list.");
				System.out.println();
				break;
			case 2:
				keyboard.nextLine();
				System.out.println("Enter new name: ");
				a.setName(keyboard.nextLine());
				break;
			case 4:
				testLibrary.removeArtist(a);
				break;
			default:
				return -1;
			}
		}

		else {
			keyboard.nextLine();
			System.out.println("Enter name of new artist: ");
			testLibrary.addArtist(keyboard.nextLine());
		}

		return 0;

	}


	public static int albumMenu() {
		SortedSet<Album> albums = testLibrary.getAllAlbums();

		if (albums == null) {
			System.out.println("**WARNING! Album list is null.  Have you implemented getAllAlbums()?");
			System.out.println();
			return -1;
		}
		
		System.out.println("Viewing all albums in library...");
		int i = 1;
		for (Album a : albums) {
			System.out.println(i++ + ".  " + a.getTitle());
		}
		System.out.println("End of listing.");

		System.out.println();
		System.out.println("Menu: ");
		System.out.println("1. View all songs in album");
		System.out.println("2. Change name of album");
		System.out.println("3. Add album to library");
		System.out.println("4. Remove album from library");
		System.out.println("5. Back to main menu");
		System.out.print("-->");
		int input = keyboard.nextInt();

		if (!(input >= 1 && input <= 5))
			return 0;

		// back to main menu
		if (input == 5) 
			return -1;

		// existing song operations
		else if (input != 3) {
			System.out.println("Enter album number: ");
			int albumNo = keyboard.nextInt();
			Album[] temp = new Album[albums.size()];
			Album a = albums.toArray(temp)[albumNo-1];
			switch (input) {
			case 1:
				System.out.println("Songs in album \"" + a.getTitle() + "\": (use song menu to edit all song details)");
				for (Song s : a.getSongs()) {
					System.out.println("Track " + s.getTrack() + ". " + s.getTitle());
				}
				System.out.println("End of listing");
				System.out.println();
				break;
			case 2:
				if (a == testLibrary.getDefaultAlbum()) {
					System.out.println("You cannot change the name of the default album.");
					System.out.println();
					break;
				}
				keyboard.nextLine();
				System.out.println("Enter new name: ");
				a.setTitle(keyboard.nextLine());
				break;
			case 4:
				testLibrary.removeAlbum(a);
				break;
			default:
				return -1;
			}
		}

		else {
			keyboard.nextLine();
			System.out.println("Enter name of new album: ");
			testLibrary.addAlbum(keyboard.nextLine());
		}

		return 0;

	}

	public static void playPause() {
		if (currentSong == null) {
			System.out.println("Please toggle a song for playback first.");
		}
		else {
			if (playerMP3.isPaused())
				playerMP3.resume();
			else if (playerMP3.isPlaying())
				playerMP3.pause();
			else
				playerMP3.play();
		}
	}

	public static void stop() {
		if (playerMP3 == null) {
			System.out.println("Nothing playing!");
		}
		else {
			playerMP3.stop();
		}
	}

	public static void main(String[] args) {

		mainMenu();

	}	
}
